from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIGroup
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class RbacAuthorizationApi:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def get_api_group(self, **kwargs: Any) -> Awaitable[V1APIGroup]: ...
    def get_api_group_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIGroup, int, CIMultiDictProxy]]: ...



# Copyright 2021 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import abstractmethod

from kubernetes_asyncio.leaderelection.leaderelectionrecord import (
    LeaderElectionRecord,
)


class BaseLock:
    def __init__(self, name: str, namespace: str, identity: str) -> None:
        self.name = name
        self.namespace = namespace
        self.identity = str(identity)

    # get returns the election record from a ConfigMap Annotation
    @abstractmethod
    async def get(
        self, name: str, namespace: str
    ) -> tuple[bool, LeaderElectionRecord] | tuple[bool, Exception] | tuple[bool, None]:
        """
        :param name: Name of the configmap object information to get
        :param namespace: Namespace in which the configmap object is to be searched
        :return: 'True, election record' if object found else 'False, exception response'
        """
        ...

    @abstractmethod
    async def create(
        self, name: str, namespace: str, election_record: LeaderElectionRecord
    ) -> bool:
        """
        :param electionRecord: Annotation string
        :param name: Name of the configmap object to be created
        :param namespace: Namespace in which the configmap object is to be created
        :return: 'True' if object is created else 'False' if failed
        """
        ...

    @abstractmethod
    async def update(
        self, name: str, namespace: str, updated_record: LeaderElectionRecord
    ) -> bool:
        """
        :param name: name of the lock to be updated
        :param namespace: namespace the lock is in
        :param updated_record: the updated election record
        :return: True if update is successful False if it fails
        """
        ...



from kubernetes_asyncio.client.models import V1beta2Counter
from typing import Any

class V1beta2DeviceCounterConsumption:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, counter_set: str | None = None, counters: dict[str, V1beta2Counter] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def counter_set(self) -> str: ...
    @counter_set.setter
    def counter_set(self, counter_set: str) -> None: ...
    @property
    def counters(self) -> dict[str, V1beta2Counter]: ...
    @counters.setter
    def counters(self, counters: dict[str, V1beta2Counter]) -> None: ...



from kubernetes_asyncio.client.models import V1CustomResourceColumnDefinition
from kubernetes_asyncio.client.models import V1CustomResourceSubresources
from kubernetes_asyncio.client.models import V1CustomResourceValidation
from kubernetes_asyncio.client.models import V1SelectableField
from typing import Any

class V1CustomResourceDefinitionVersion:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, additional_printer_columns: list[V1CustomResourceColumnDefinition] | None = None, deprecated: bool | None = None, deprecation_warning: str | None = None, name: str | None = None, schema: V1CustomResourceValidation | None = None, selectable_fields: list[V1SelectableField] | None = None, served: bool | None = None, storage: bool | None = None, subresources: V1CustomResourceSubresources | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def additional_printer_columns(self) -> list[V1CustomResourceColumnDefinition]: ...
    @additional_printer_columns.setter
    def additional_printer_columns(self, additional_printer_columns: list[V1CustomResourceColumnDefinition]) -> None: ...
    @property
    def deprecated(self) -> bool: ...
    @deprecated.setter
    def deprecated(self, deprecated: bool) -> None: ...
    @property
    def deprecation_warning(self) -> str: ...
    @deprecation_warning.setter
    def deprecation_warning(self, deprecation_warning: str) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def schema(self) -> V1CustomResourceValidation: ...
    @schema.setter
    def schema(self, schema: V1CustomResourceValidation) -> None: ...
    @property
    def selectable_fields(self) -> list[V1SelectableField]: ...
    @selectable_fields.setter
    def selectable_fields(self, selectable_fields: list[V1SelectableField]) -> None: ...
    @property
    def served(self) -> bool: ...
    @served.setter
    def served(self, served: bool) -> None: ...
    @property
    def storage(self) -> bool: ...
    @storage.setter
    def storage(self, storage: bool) -> None: ...
    @property
    def subresources(self) -> V1CustomResourceSubresources: ...
    @subresources.setter
    def subresources(self, subresources: V1CustomResourceSubresources) -> None: ...



from datetime import datetime
from kubernetes_asyncio.client.models import V1JobCondition
from kubernetes_asyncio.client.models import V1UncountedTerminatedPods
from typing import Any

class V1JobStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, active: int | None = None, completed_indexes: str | None = None, completion_time: datetime | None = None, conditions: list[V1JobCondition] | None = None, failed: int | None = None, failed_indexes: str | None = None, ready: int | None = None, start_time: datetime | None = None, succeeded: int | None = None, terminating: int | None = None, uncounted_terminated_pods: V1UncountedTerminatedPods | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def active(self) -> int: ...
    @active.setter
    def active(self, active: int) -> None: ...
    @property
    def completed_indexes(self) -> str: ...
    @completed_indexes.setter
    def completed_indexes(self, completed_indexes: str) -> None: ...
    @property
    def completion_time(self) -> datetime: ...
    @completion_time.setter
    def completion_time(self, completion_time: datetime) -> None: ...
    @property
    def conditions(self) -> list[V1JobCondition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1JobCondition]) -> None: ...
    @property
    def failed(self) -> int: ...
    @failed.setter
    def failed(self, failed: int) -> None: ...
    @property
    def failed_indexes(self) -> str: ...
    @failed_indexes.setter
    def failed_indexes(self, failed_indexes: str) -> None: ...
    @property
    def ready(self) -> int: ...
    @ready.setter
    def ready(self, ready: int) -> None: ...
    @property
    def start_time(self) -> datetime: ...
    @start_time.setter
    def start_time(self, start_time: datetime) -> None: ...
    @property
    def succeeded(self) -> int: ...
    @succeeded.setter
    def succeeded(self, succeeded: int) -> None: ...
    @property
    def terminating(self) -> int: ...
    @terminating.setter
    def terminating(self, terminating: int) -> None: ...
    @property
    def uncounted_terminated_pods(self) -> V1UncountedTerminatedPods: ...
    @uncounted_terminated_pods.setter
    def uncounted_terminated_pods(self, uncounted_terminated_pods: V1UncountedTerminatedPods) -> None: ...



from typing import Any

class V1PodFailurePolicyOnExitCodesRequirement:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, container_name: str | None = None, operator: str | None = None, values: list[int] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def container_name(self) -> str: ...
    @container_name.setter
    def container_name(self, container_name: str) -> None: ...
    @property
    def operator(self) -> str: ...
    @operator.setter
    def operator(self, operator: str) -> None: ...
    @property
    def values(self) -> list[int]: ...
    @values.setter
    def values(self, values: list[int]) -> None: ...



from kubernetes_asyncio.client.models import V1NodeConfigSource
from typing import Any

class V1NodeConfigStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, active: V1NodeConfigSource | None = None, assigned: V1NodeConfigSource | None = None, error: str | None = None, last_known_good: V1NodeConfigSource | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def active(self) -> V1NodeConfigSource: ...
    @active.setter
    def active(self, active: V1NodeConfigSource) -> None: ...
    @property
    def assigned(self) -> V1NodeConfigSource: ...
    @assigned.setter
    def assigned(self, assigned: V1NodeConfigSource) -> None: ...
    @property
    def error(self) -> str: ...
    @error.setter
    def error(self, error: str) -> None: ...
    @property
    def last_known_good(self) -> V1NodeConfigSource: ...
    @last_known_good.setter
    def last_known_good(self, last_known_good: V1NodeConfigSource) -> None: ...



from kubernetes_asyncio.client.models import V1NetworkPolicyPeer
from kubernetes_asyncio.client.models import V1NetworkPolicyPort
from typing import Any

class V1NetworkPolicyIngressRule:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, _from: list[V1NetworkPolicyPeer] | None = None, ports: list[V1NetworkPolicyPort] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def _from(self) -> list[V1NetworkPolicyPeer]: ...
    @_from.setter
    def _from(self, _from: list[V1NetworkPolicyPeer]) -> None: ...
    @property
    def ports(self) -> list[V1NetworkPolicyPort]: ...
    @ports.setter
    def ports(self, ports: list[V1NetworkPolicyPort]) -> None: ...



from kubernetes_asyncio.client.models import V2MetricIdentifier
from kubernetes_asyncio.client.models import V2MetricValueStatus
from typing import Any

class V2PodsMetricStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, current: V2MetricValueStatus | None = None, metric: V2MetricIdentifier | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def current(self) -> V2MetricValueStatus: ...
    @current.setter
    def current(self, current: V2MetricValueStatus) -> None: ...
    @property
    def metric(self) -> V2MetricIdentifier: ...
    @metric.setter
    def metric(self, metric: V2MetricIdentifier) -> None: ...



from typing import Any

class V1LimitRangeItem:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, default: dict[str, str] | None = None, default_request: dict[str, str] | None = None, max: dict[str, str] | None = None, max_limit_request_ratio: dict[str, str] | None = None, min: dict[str, str] | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def default(self) -> dict[str, str]: ...
    @default.setter
    def default(self, default: dict[str, str]) -> None: ...
    @property
    def default_request(self) -> dict[str, str]: ...
    @default_request.setter
    def default_request(self, default_request: dict[str, str]) -> None: ...
    @property
    def max(self) -> dict[str, str]: ...
    @max.setter
    def max(self, max: dict[str, str]) -> None: ...
    @property
    def max_limit_request_ratio(self) -> dict[str, str]: ...
    @max_limit_request_ratio.setter
    def max_limit_request_ratio(self, max_limit_request_ratio: dict[str, str]) -> None: ...
    @property
    def min(self) -> dict[str, str]: ...
    @min.setter
    def min(self, min: dict[str, str]) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



# Copyright 2016 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import atexit
import base64
import copy
import datetime
import json
import logging
import os
import pathlib
import platform
import tempfile
from typing import Any, cast

import yaml

from kubernetes_asyncio.client import ApiClient, Configuration
from kubernetes_asyncio.config.config_exception import ConfigException
from kubernetes_asyncio.config.dateutil import UTC, parse_rfc3339
from kubernetes_asyncio.config.exec_provider import ExecProvider
from kubernetes_asyncio.config.google_auth import google_auth_credentials
from kubernetes_asyncio.config.openid import OpenIDRequestor

EXPIRY_SKEW_PREVENTION_DELAY = datetime.timedelta(minutes=5)
KUBE_CONFIG_DEFAULT_LOCATION = os.environ.get(
    "KUBECONFIG", (pathlib.Path.home() / ".kube/config").as_posix()
)
ENV_KUBECONFIG_PATH_SEPARATOR = ";" if platform.system() == "Windows" else ":"
PROVIDER_TYPE_OIDC = "oidc"
_temp_files: dict[str, str] = {}
logger = logging.getLogger(__name__)


def _cleanup_temp_files() -> None:
    global _temp_files
    for temp_file in _temp_files.values():
        try:
            os.remove(temp_file)
        except OSError:
            pass
    _temp_files = {}


def _is_expired(expiry: str | datetime.datetime) -> bool:
    return (
        parse_rfc3339(expiry) - EXPIRY_SKEW_PREVENTION_DELAY
    ) <= datetime.datetime.utcnow().replace(tzinfo=UTC)


class FileOrData:
    """Utility class to read content of obj[%data_key_name] or file's
    content of obj[%file_key_name] and represent it as file or data.
    Note that the data is preferred. The obj[%file_key_name] will be used iff
    obj['%data_key_name'] is not set or empty. Assumption is file content is
    raw data and data field is base64 string. The assumption can be changed
    with base64_file_content flag. If set to False, the content of the file
    will assumed to be base64 and read as is. The default True value will
    result in base64 encode of the file content after read."""

    def __init__(
        self,
        obj,
        file_key_name,
        data_key_name=None,
        file_base_path="",
        base64_file_content=True,
        temp_file_path=None,
    ) -> None:
        if not data_key_name:
            data_key_name = file_key_name + "-data"
        self._file = None
        self._data = None
        self._base64_file_content = base64_file_content
        self._temp_file_path = temp_file_path
        if temp_file_path:
            os.makedirs(name=temp_file_path, exist_ok=True)
        if data_key_name in obj:
            self._data = obj[data_key_name]
        elif file_key_name in obj:
            self._file = os.path.normpath(
                os.path.join(file_base_path, obj[file_key_name])
            )

    def _create_temp_file_with_content(self, content) -> str:
        if len(_temp_files) == 0:
            atexit.register(_cleanup_temp_files)
        # Because we may change context several times, try to remember files we
        # created and reuse them at a small memory cost.
        content_key = str(content)
        if content_key in _temp_files:
            return _temp_files[content_key]
        _, name = tempfile.mkstemp(dir=self._temp_file_path)
        _temp_files[content_key] = name
        with open(name, "wb") as fd:
            fd.write(content.encode() if isinstance(content, str) else content)
        return name

    def as_file(self) -> str | None:
        """If obj[%data_key_name] exists, return name of a file with base64
        decoded obj[%data_key_name] content otherwise obj[%file_key_name]."""
        if not self._file and self._data:
            if self._base64_file_content:
                if isinstance(self._data, str):
                    content = self._data.encode()
                else:
                    content = self._data
                self._file = self._create_temp_file_with_content(
                    base64.standard_b64decode(content)
                )
            else:
                self._file = self._create_temp_file_with_content(self._data)
        if self._file and not os.path.isfile(self._file):
            raise ConfigException("File does not exists: %s" % self._file)
        return self._file

    def as_data(self) -> str | None:
        """If obj[%data_key_name] exists, Return obj[%data_key_name] otherwise
        base64 encoded string of obj[%file_key_name] file content."""
        if not self._data and self._file:
            with open(self._file) as f:
                if self._base64_file_content:
                    self._data = bytes.decode(
                        base64.standard_b64encode(str.encode(f.read()))
                    )
                else:
                    self._data = f.read()
        return self._data


class KubeConfigLoader(object):
    def __init__(
        self,
        config_dict: Any,
        active_context=None,
        get_google_credentials=None,
        config_base_path: str | None = "",
        config_persister=None,
        temp_file_path=None,
    ):
        if isinstance(config_dict, ConfigNode):
            self._config = config_dict
        else:
            self._config = ConfigNode("kube-config", config_dict)

        self._current_context = None
        self._user: ConfigNode | None = None
        self._cluster = None
        self.provider = None
        self.set_active_context(active_context)
        self._config_base_path = config_base_path
        self._config_persister = config_persister
        self._temp_file_path = temp_file_path
        if get_google_credentials:
            self._get_google_credentials = get_google_credentials
        else:
            self._get_google_credentials = None
        self.token: str
        self.exec_plugin_expiry: datetime.datetime

    def set_active_context(self, context_name=None):
        if context_name is None:
            context_name = self._config["current-context"]
        self._current_context = self._config["contexts"].get_with_name(context_name)
        if (
            self._current_context
            and self._current_context["context"].safe_get("user")
            and self._config.safe_get("users")
        ):
            user = self._config["users"].get_with_name(
                self._current_context["context"]["user"], safe=True
            )
            if user:
                self._user = user["user"]
            else:
                self._user = None
        else:
            self._user = None

        assert self._current_context
        cluster = self._config["clusters"].get_with_name(
            self._current_context["context"]["cluster"]
        )
        assert cluster
        self._cluster = cluster["cluster"]
        if (
            self._user is not None
            and "auth-provider" in self._user
            and "name" in self._user["auth-provider"]
        ):
            self.provider = self._user["auth-provider"]["name"]

        logger.debug(
            "kubeconfig loader - current-context %s, cluster %s, user %s, provider %s",
            context_name,
            self._current_context["context"]["cluster"],
            self._current_context["context"].safe_get("user"),
            self.provider,
        )

    async def _load_authentication(self) -> None:
        """Read authentication from kube-config user section if exists.

        This function goes through various authentication methods in user
        section of kube-config and stops if it finds a valid authentication
        method. The order of authentication methods is:

            1. GCP auth-provider
            2. token field (point to a token file)
            3. oidc auth-provider
            4. exec provided plugin
            5. username/password
        """

        if not self._user:
            logger.debug("No user section in current context.")
            return

        if self.provider == "gcp":
            await self.load_gcp_token()
            return

        if self.provider == PROVIDER_TYPE_OIDC:
            await self._load_oid_token()
            return

        if "exec" in self._user:
            logger.debug("Try to use exec provider")
            res_exec_plugin = await self.load_from_exec_plugin()
            if res_exec_plugin:
                return

        logger.debug("Try to load user token")
        if self._load_user_token():
            return

        logger.debug("Try to use username and password")
        self._load_user_pass_token()

    async def load_gcp_token(self) -> str:
        assert self._user
        if "config" not in self._user["auth-provider"]:
            self._user["auth-provider"].value["config"] = {}

        config = self._user["auth-provider"]["config"]

        if ("access-token" not in config) or (
            "expiry" in config and _is_expired(str(config["expiry"]))
        ):
            if self._get_google_credentials is not None:
                if asyncio.iscoroutinefunction(self._get_google_credentials):
                    credentials = await self._get_google_credentials()
                else:
                    credentials = self._get_google_credentials()
            else:
                credentials = await google_auth_credentials(config)
            config.value["access-token"] = credentials.token
            config.value["expiry"] = credentials.expiry
            if self._config_persister:
                self._config_persister(self._config.value)

        self.token = "Bearer %s" % config["access-token"]
        return self.token

    async def _load_oid_token(self) -> str:
        assert self._user
        provider = self._user["auth-provider"]

        if "config" not in provider:
            raise ValueError("oidc: missing configuration")

        if "id-token" not in provider["config"]:
            await self._refresh_oidc(provider)

            self.token = "Bearer {}".format(provider["config"]["id-token"])
            return self.token

        id_token = provider["config"]["id-token"]
        assert isinstance(id_token, str)
        parts = id_token.split(".")

        if len(parts) != 3:
            raise ValueError("oidc: JWT tokens should contain 3 period-delimited parts")

        id_token = parts[1]
        # Re-pad the unpadded JWT token
        id_token += (4 - len(id_token) % 4) * "="
        jwt_attributes = json.loads(base64.b64decode(id_token).decode("utf8"))
        expires = jwt_attributes.get("exp")

        if expires is not None and _is_expired(
            datetime.datetime.utcfromtimestamp(expires)
        ):
            await self._refresh_oidc(provider)

        self.token = "Bearer {}".format(provider["config"]["id-token"])
        return self.token

    async def _refresh_oidc(self, provider) -> None:
        if "refresh-token" not in provider["config"]:
            raise ConfigException(
                "oidc: No valid id-token, and cannot refresh without refresh-token"
            )

        with tempfile.NamedTemporaryFile(delete=True) as certfile:
            ssl_ca_cert = None
            cert_auth_data = self._retrieve_oidc_cacert(provider)
            if cert_auth_data is not None:
                certfile.write(cert_auth_data)
                certfile.flush()
                ssl_ca_cert = certfile.name

            requestor = OpenIDRequestor(
                provider["config"]["client-id"],
                provider["config"]["client-secret"],
                provider["config"]["idp-issuer-url"],
                ssl_ca_cert,
            )

            resp = await requestor.refresh_token(provider["config"]["refresh-token"])

            provider["config"].value["id-token"] = resp["id_token"]
            provider["config"].value["refresh-token"] = resp["refresh_token"]

            if self._config_persister:
                self._config_persister(self._config.value)

    def _retrieve_oidc_cacert(self, provider) -> bytes | None:
        if "idp-certificate-authority-data" in provider["config"]:
            return base64.b64decode(
                provider["config"]["idp-certificate-authority-data"]
            )

        return None

    async def load_from_exec_plugin(self) -> bool:
        try:
            if hasattr(self, "exec_plugin_expiry") and not _is_expired(
                cast(datetime.datetime, self.exec_plugin_expiry)
            ):
                return True
            assert self._cluster
            base_path = self._get_base_path(self._cluster.path)
            assert self._user
            status = await ExecProvider(self._user["exec"]).run()
            if "token" in status:
                self.token = "Bearer %s" % status["token"]
                if "expirationTimestamp" in status:
                    self.exec_plugin_expiry = parse_rfc3339(
                        status["expirationTimestamp"]
                    )
            elif "clientCertificateData" in status:
                # https://kubernetes.io/docs/reference/access-authn-authz/authentication/#input-and-output-formats
                # Plugin has provided certificates instead of a token.
                if "clientKeyData" not in status:
                    logger.error("exec: missing clientKeyData field in plugin output")
                    return False
                self.cert_file = FileOrData(
                    status,
                    None,
                    data_key_name="clientCertificateData",
                    file_base_path=base_path,
                    base64_file_content=False,
                    temp_file_path=self._temp_file_path,
                ).as_file()
                self.key_file = FileOrData(
                    status,
                    None,
                    data_key_name="clientKeyData",
                    file_base_path=base_path,
                    base64_file_content=False,
                    temp_file_path=self._temp_file_path,
                ).as_file()
            else:
                logger.error(
                    "exec: missing token or clientCertificateData "
                    "field in plugin output"
                )
            return True
        except Exception as e:
            logger.error(str(e))
            raise
        return False

    def _load_user_token(self) -> bool:
        assert self._user
        base_path = self._get_base_path(self._user.path)
        token = FileOrData(
            self._user,
            "tokenFile",
            "token",
            file_base_path=base_path,
            base64_file_content=False,
        ).as_data()
        if token:
            self.token = "Bearer %s" % token
            return True
        return False

    def _load_user_pass_token(self) -> bool:
        assert self._user
        if "username" in self._user and "password" in self._user:
            basic_auth = str(self._user["username"]) + ":" + str(self._user["password"])
            self.token = "Basic " + base64.b64encode(basic_auth.encode()).decode(
                "utf-8"
            )
            return True
        return False

    def _get_base_path(self, config_path) -> str:
        if self._config_base_path is not None:
            return self._config_base_path
        if config_path is not None:
            return os.path.abspath(os.path.dirname(config_path))
        return ""

    def _load_cluster_info(self) -> None:
        assert self._cluster
        if "server" in self._cluster:
            server = self._cluster["server"]
            assert isinstance(server, str)
            self.host = server.rstrip("/")
            if self.host.startswith("https"):
                base_path = self._get_base_path(self._cluster.path)
                self.ssl_ca_cert = FileOrData(
                    self._cluster,
                    "certificate-authority",
                    file_base_path=base_path,
                    temp_file_path=self._temp_file_path,
                ).as_file()
                if "cert_file" not in self.__dict__:
                    # cert_file could have been provided by
                    # _load_from_exec_plugin; only load from the _user
                    # section if we need it.
                    self.cert_file = FileOrData(
                        self._user,
                        "client-certificate",
                        file_base_path=base_path,
                        temp_file_path=self._temp_file_path,
                    ).as_file()
                    self.key_file = FileOrData(
                        self._user,
                        "client-key",
                        file_base_path=base_path,
                        temp_file_path=self._temp_file_path,
                    ).as_file()
        if "insecure-skip-tls-verify" in self._cluster:
            self.verify_ssl = not self._cluster["insecure-skip-tls-verify"]
        if "tls-server-name" in self._cluster:
            self.tls_server_name = self._cluster["tls-server-name"]
        if "proxy-url" in self._cluster:
            self.proxy = self._cluster["proxy-url"]

    def _set_config(self, client_configuration) -> None:
        if hasattr(self, "token"):
            client_configuration.api_key["BearerToken"] = self.token

        # copy these keys directly from self to configuration object
        keys = [
            "host",
            "ssl_ca_cert",
            "cert_file",
            "key_file",
            "verify_ssl",
            "tls_server_name",
            "proxy",
        ]
        for key in keys:
            if key in self.__dict__:
                setattr(client_configuration, key, getattr(self, key))

    async def load_and_set(self, client_configuration) -> None:
        await self._load_authentication()
        self._load_cluster_info()
        self._set_config(client_configuration)

    def list_contexts(self) -> list[Any]:
        contexts = self._config["contexts"]
        assert contexts
        return [context.value for context in cast(list, contexts)]

    @property
    def current_context(self) -> Any | None:
        if self._current_context:
            return self._current_context.value
        return None


class ConfigNode:
    """Remembers each config key's path and construct a relevant exception
    message in case of missing keys. The assumption is all access keys are
    present in a well-formed kube-config."""

    def __init__(self, name: str, value: Any, path: str | None = None) -> None:
        self.name: str = name
        self.value: Any = value
        self.path: str | None = path

    def __contains__(self, key):
        return key in self.value

    def __len__(self) -> int:
        return len(self.value)

    def safe_get(self, key: str | int) -> Any:
        if isinstance(self.value, list):
            if isinstance(key, int):
                return self.value[key]
        elif isinstance(self.value, dict):
            if key in self.value:
                return self.value[key]
        return None

    def __getitem__(self, key: str | int) -> "ConfigNode | Any":
        v = self.safe_get(key)
        if v is None:
            raise ConfigException(
                "Invalid kube-config file. Expected key %s in %s" % (key, self.name)
            )
        if isinstance(v, dict) or isinstance(v, list):
            return ConfigNode("%s/%s" % (self.name, key), v, self.path)
        else:
            return v

    def get_with_name(self, name, safe=False) -> "ConfigNode | None":
        if not isinstance(self.value, list):
            raise ConfigException(
                "Invalid kube-config file. Expected %s to be a list" % self.name
            )
        result = None
        for v in self.value:
            if "name" not in v:
                raise ConfigException(
                    "Invalid kube-config file. "
                    "Expected all values in %s list to have 'name' key" % self.name
                )
            if v["name"] == name:
                if result is None:
                    result = v
                else:
                    raise ConfigException(
                        "Invalid kube-config file. "
                        "Expected only one object with name %s in %s list"
                        % (name, self.name)
                    )
        if result is not None:
            if isinstance(result, ConfigNode):
                return result
            else:
                return ConfigNode("%s[name=%s]" % (self.name, name), result, self.path)
        if safe:
            return None
        raise ConfigException(
            "Invalid kube-config file. "
            "Expected object with name %s in %s list" % (name, self.name)
        )


class KubeConfigMerger:
    """Reads and merges configuration from one or more kube-config's.
    The propery `config` can be passed to the KubeConfigLoader as config_dict.
    It uses a path attribute from ConfigNode to store the path to kubeconfig.
    This path is required to load certs from relative paths.
    A method `save_changes` updates changed kubeconfig's (it compares current
    state of dicts with).
    """

    def __init__(self, paths: str) -> None:
        self.paths = []
        self.config_files: dict[str, Any] = {}
        self.config_merged: ConfigNode | None = None

        file_loaded = False
        for path in paths.split(ENV_KUBECONFIG_PATH_SEPARATOR):
            if path:
                path = os.path.expanduser(path)
                if os.path.exists(path):
                    self.paths.append(path)
                    self.load_config(path)
                    file_loaded = True
        self.config_saved = copy.deepcopy(self.config_files)
        if not file_loaded:
            logger.warning("Config not found: %s", paths)

    @property
    def config(self) -> ConfigNode | None:
        return self.config_merged

    def load_config(self, path) -> None:
        with open(path) as f:
            config = yaml.safe_load(f)

        if self.config_merged is None:
            config_merged = copy.deepcopy(config)
            for item in ("clusters", "contexts", "users"):
                config_merged[item] = []
            self.config_merged = ConfigNode(path, config_merged, path)

        for item in ("clusters", "contexts", "users"):
            self._merge(item, config.get(item, []) or [], path)

        if "current-context" in config:
            self.config_merged.value["current-context"] = config["current-context"]

        self.config_files[path] = config

    def _merge(self, item, add_cfg, path) -> None:
        assert self.config_merged
        for new_item in add_cfg:
            for exists in self.config_merged.value[item]:
                if exists["name"] == new_item["name"]:
                    break
            else:
                self.config_merged.value[item].append(
                    ConfigNode("{}/{}".format(path, new_item), new_item, path)
                )

    def save_changes(self) -> None:
        for path in self.paths:
            if self.config_saved[path] != self.config_files[path]:
                self.save_config(path)
        self.config_saved = copy.deepcopy(self.config_files)

    def save_config(self, path) -> None:
        with open(path, "w") as f:
            yaml.safe_dump(self.config_files[path], f, default_flow_style=False)


def _get_kube_config_loader_for_yaml_file(
    filename, persist_config=False, **kwargs
) -> KubeConfigLoader:
    kcfg = KubeConfigMerger(filename)
    if persist_config and "config_persister" not in kwargs:
        kwargs["config_persister"] = kcfg.save_changes

    return KubeConfigLoader(config_dict=kcfg.config, config_base_path=None, **kwargs)


def list_kube_config_contexts(
    config_file: str | None = None,
) -> tuple[list[Any], Any | None]:
    if config_file is None:
        config_file = os.path.expanduser(KUBE_CONFIG_DEFAULT_LOCATION)

    loader = _get_kube_config_loader_for_yaml_file(config_file)
    return loader.list_contexts(), loader.current_context


async def load_kube_config(
    config_file=None,
    context=None,
    client_configuration=None,
    persist_config=True,
    temp_file_path=None,
) -> KubeConfigLoader:
    """Loads authentication and cluster information from kube-config file
    and stores them in kubernetes.client.configuration.

    :param config_file: Name of the kube-config file.
    :param context: set the active context. If is set to None, current_context
        from config file will be used.
    :param client_configuration: The kubernetes.client.Configuration to
        set configs to.
    :param persist_config: If True, config file will be updated when changed
        (e.g GCP token refresh).
    :param temp_file_path: directory where temp files are stored
        (default - system temp dir).
    """

    if config_file is None:
        config_file = KUBE_CONFIG_DEFAULT_LOCATION

    loader = _get_kube_config_loader_for_yaml_file(
        config_file,
        active_context=context,
        persist_config=persist_config,
        temp_file_path=temp_file_path,
    )
    if client_configuration is None:
        config = type.__call__(Configuration)
        await loader.load_and_set(config)
        Configuration.set_default(config)
    else:
        await loader.load_and_set(client_configuration)

    return loader


async def load_kube_config_from_dict(
    config_dict, context=None, client_configuration=None, temp_file_path=None
) -> KubeConfigLoader:
    """Loads authentication and cluster information from config_dict
    and stores them in kubernetes.client.configuration.

    :param config_dict: Takes the config file as a dict.
    :param context: set the active context. If is set to None, current_context
        from config file will be used.
    :param client_configuration: The kubernetes_asyncio.client.Configuration to
        set configs to.
    :param temp_file_path: directory where temp files are stored
        (default - system temp dir).
    """

    loader = KubeConfigLoader(
        config_dict=config_dict,
        config_base_path=None,
        active_context=context,
        temp_file_path=temp_file_path,
    )

    if client_configuration is None:
        config = type.__call__(Configuration)
        await loader.load_and_set(config)
        Configuration.set_default(config)
    else:
        await loader.load_and_set(client_configuration)

    return loader


async def refresh_token(loader, client_configuration=None, interval=60) -> None:
    """Refresh token if necessary, updates the token in client configurarion

    :param loader: KubeConfigLoader returned by load_kube_config
    :param client_configuration: The kubernetes.client.Configuration to
            set configs to.
    :param interval: how often check if token is up-to-date

    """

    if client_configuration is None:
        raise NotImplementedError

    if loader.provider == "gcp":
        while 1:
            await asyncio.sleep(interval)
            await loader.load_gcp_token()
            client_configuration.api_key["BearerToken"] = loader.token
    elif "exec" in loader._user:
        while 1:
            await asyncio.sleep(interval)
            await loader.load_from_exec_plugin()
            client_configuration.api_key["BearerToken"] = loader.token


async def new_client_from_config(
    config_file=None, context=None, persist_config=True, temp_file_path=None
) -> ApiClient:
    """Loads configuration the same as load_kube_config but returns an ApiClient
    to be used with any API object. This will allow the caller to concurrently
    talk with multiple clusters."""
    client_config = type.__call__(Configuration)

    await load_kube_config(
        config_file=config_file,
        context=context,
        client_configuration=client_config,
        persist_config=persist_config,
        temp_file_path=temp_file_path,
    )

    return ApiClient(configuration=client_config)


async def new_client_from_config_dict(
    config_dict=None, context=None, temp_file_path=None
) -> ApiClient:
    """Loads configuration the same as load_kube_config_dict but returns an ApiClient
    to be used with any API object. This will allow the caller to concurrently
    talk with multiple clusters."""
    client_config = type.__call__(Configuration)

    await load_kube_config_from_dict(
        config_dict=config_dict,
        context=context,
        client_configuration=client_config,
        temp_file_path=temp_file_path,
    )

    return ApiClient(configuration=client_config)



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1Status
from kubernetes_asyncio.client.models import V1beta1MutatingAdmissionPolicy
from kubernetes_asyncio.client.models import V1beta1MutatingAdmissionPolicyBinding
from kubernetes_asyncio.client.models import V1beta1MutatingAdmissionPolicyBindingList
from kubernetes_asyncio.client.models import V1beta1MutatingAdmissionPolicyList
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class AdmissionregistrationV1beta1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_mutating_admission_policy(self, body: V1beta1MutatingAdmissionPolicy, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicy]: ...
    def create_mutating_admission_policy_binding(self, body: V1beta1MutatingAdmissionPolicyBinding, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicyBinding]: ...
    def create_mutating_admission_policy_binding_with_http_info(self, body: V1beta1MutatingAdmissionPolicyBinding, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicyBinding, int, CIMultiDictProxy]]: ...
    def create_mutating_admission_policy_with_http_info(self, body: V1beta1MutatingAdmissionPolicy, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def delete_collection_mutating_admission_policy(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_mutating_admission_policy_binding(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_mutating_admission_policy_binding_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_mutating_admission_policy_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_mutating_admission_policy(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_mutating_admission_policy_binding(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_mutating_admission_policy_binding_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_mutating_admission_policy_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_mutating_admission_policy(self, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicyList]: ...
    def list_mutating_admission_policy_binding(self, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicyBindingList]: ...
    def list_mutating_admission_policy_binding_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicyBindingList, int, CIMultiDictProxy]]: ...
    def list_mutating_admission_policy_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicyList, int, CIMultiDictProxy]]: ...
    def patch_mutating_admission_policy(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicy]: ...
    def patch_mutating_admission_policy_binding(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicyBinding]: ...
    def patch_mutating_admission_policy_binding_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicyBinding, int, CIMultiDictProxy]]: ...
    def patch_mutating_admission_policy_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def read_mutating_admission_policy(self, name: str, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicy]: ...
    def read_mutating_admission_policy_binding(self, name: str, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicyBinding]: ...
    def read_mutating_admission_policy_binding_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicyBinding, int, CIMultiDictProxy]]: ...
    def read_mutating_admission_policy_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def replace_mutating_admission_policy(self, name: str, body: V1beta1MutatingAdmissionPolicy, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicy]: ...
    def replace_mutating_admission_policy_binding(self, name: str, body: V1beta1MutatingAdmissionPolicyBinding, **kwargs: Any) -> Awaitable[V1beta1MutatingAdmissionPolicyBinding]: ...
    def replace_mutating_admission_policy_binding_with_http_info(self, name: str, body: V1beta1MutatingAdmissionPolicyBinding, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicyBinding, int, CIMultiDictProxy]]: ...
    def replace_mutating_admission_policy_with_http_info(self, name: str, body: V1beta1MutatingAdmissionPolicy, **kwargs: Any) -> Awaitable[tuple[V1beta1MutatingAdmissionPolicy, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1AWSElasticBlockStoreVolumeSource
from kubernetes_asyncio.client.models import V1AzureDiskVolumeSource
from kubernetes_asyncio.client.models import V1AzureFileVolumeSource
from kubernetes_asyncio.client.models import V1CSIVolumeSource
from kubernetes_asyncio.client.models import V1CephFSVolumeSource
from kubernetes_asyncio.client.models import V1CinderVolumeSource
from kubernetes_asyncio.client.models import V1ConfigMapVolumeSource
from kubernetes_asyncio.client.models import V1DownwardAPIVolumeSource
from kubernetes_asyncio.client.models import V1EmptyDirVolumeSource
from kubernetes_asyncio.client.models import V1EphemeralVolumeSource
from kubernetes_asyncio.client.models import V1FCVolumeSource
from kubernetes_asyncio.client.models import V1FlexVolumeSource
from kubernetes_asyncio.client.models import V1FlockerVolumeSource
from kubernetes_asyncio.client.models import V1GCEPersistentDiskVolumeSource
from kubernetes_asyncio.client.models import V1GitRepoVolumeSource
from kubernetes_asyncio.client.models import V1GlusterfsVolumeSource
from kubernetes_asyncio.client.models import V1HostPathVolumeSource
from kubernetes_asyncio.client.models import V1ISCSIVolumeSource
from kubernetes_asyncio.client.models import V1ImageVolumeSource
from kubernetes_asyncio.client.models import V1NFSVolumeSource
from kubernetes_asyncio.client.models import V1PersistentVolumeClaimVolumeSource
from kubernetes_asyncio.client.models import V1PhotonPersistentDiskVolumeSource
from kubernetes_asyncio.client.models import V1PortworxVolumeSource
from kubernetes_asyncio.client.models import V1ProjectedVolumeSource
from kubernetes_asyncio.client.models import V1QuobyteVolumeSource
from kubernetes_asyncio.client.models import V1RBDVolumeSource
from kubernetes_asyncio.client.models import V1ScaleIOVolumeSource
from kubernetes_asyncio.client.models import V1SecretVolumeSource
from kubernetes_asyncio.client.models import V1StorageOSVolumeSource
from kubernetes_asyncio.client.models import V1VsphereVirtualDiskVolumeSource
from typing import Any

class V1Volume:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, aws_elastic_block_store: V1AWSElasticBlockStoreVolumeSource | None = None, azure_disk: V1AzureDiskVolumeSource | None = None, azure_file: V1AzureFileVolumeSource | None = None, cephfs: V1CephFSVolumeSource | None = None, cinder: V1CinderVolumeSource | None = None, config_map: V1ConfigMapVolumeSource | None = None, csi: V1CSIVolumeSource | None = None, downward_api: V1DownwardAPIVolumeSource | None = None, empty_dir: V1EmptyDirVolumeSource | None = None, ephemeral: V1EphemeralVolumeSource | None = None, fc: V1FCVolumeSource | None = None, flex_volume: V1FlexVolumeSource | None = None, flocker: V1FlockerVolumeSource | None = None, gce_persistent_disk: V1GCEPersistentDiskVolumeSource | None = None, git_repo: V1GitRepoVolumeSource | None = None, glusterfs: V1GlusterfsVolumeSource | None = None, host_path: V1HostPathVolumeSource | None = None, image: V1ImageVolumeSource | None = None, iscsi: V1ISCSIVolumeSource | None = None, name: str | None = None, nfs: V1NFSVolumeSource | None = None, persistent_volume_claim: V1PersistentVolumeClaimVolumeSource | None = None, photon_persistent_disk: V1PhotonPersistentDiskVolumeSource | None = None, portworx_volume: V1PortworxVolumeSource | None = None, projected: V1ProjectedVolumeSource | None = None, quobyte: V1QuobyteVolumeSource | None = None, rbd: V1RBDVolumeSource | None = None, scale_io: V1ScaleIOVolumeSource | None = None, secret: V1SecretVolumeSource | None = None, storageos: V1StorageOSVolumeSource | None = None, vsphere_volume: V1VsphereVirtualDiskVolumeSource | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def aws_elastic_block_store(self) -> V1AWSElasticBlockStoreVolumeSource: ...
    @aws_elastic_block_store.setter
    def aws_elastic_block_store(self, aws_elastic_block_store: V1AWSElasticBlockStoreVolumeSource) -> None: ...
    @property
    def azure_disk(self) -> V1AzureDiskVolumeSource: ...
    @azure_disk.setter
    def azure_disk(self, azure_disk: V1AzureDiskVolumeSource) -> None: ...
    @property
    def azure_file(self) -> V1AzureFileVolumeSource: ...
    @azure_file.setter
    def azure_file(self, azure_file: V1AzureFileVolumeSource) -> None: ...
    @property
    def cephfs(self) -> V1CephFSVolumeSource: ...
    @cephfs.setter
    def cephfs(self, cephfs: V1CephFSVolumeSource) -> None: ...
    @property
    def cinder(self) -> V1CinderVolumeSource: ...
    @cinder.setter
    def cinder(self, cinder: V1CinderVolumeSource) -> None: ...
    @property
    def config_map(self) -> V1ConfigMapVolumeSource: ...
    @config_map.setter
    def config_map(self, config_map: V1ConfigMapVolumeSource) -> None: ...
    @property
    def csi(self) -> V1CSIVolumeSource: ...
    @csi.setter
    def csi(self, csi: V1CSIVolumeSource) -> None: ...
    @property
    def downward_api(self) -> V1DownwardAPIVolumeSource: ...
    @downward_api.setter
    def downward_api(self, downward_api: V1DownwardAPIVolumeSource) -> None: ...
    @property
    def empty_dir(self) -> V1EmptyDirVolumeSource: ...
    @empty_dir.setter
    def empty_dir(self, empty_dir: V1EmptyDirVolumeSource) -> None: ...
    @property
    def ephemeral(self) -> V1EphemeralVolumeSource: ...
    @ephemeral.setter
    def ephemeral(self, ephemeral: V1EphemeralVolumeSource) -> None: ...
    @property
    def fc(self) -> V1FCVolumeSource: ...
    @fc.setter
    def fc(self, fc: V1FCVolumeSource) -> None: ...
    @property
    def flex_volume(self) -> V1FlexVolumeSource: ...
    @flex_volume.setter
    def flex_volume(self, flex_volume: V1FlexVolumeSource) -> None: ...
    @property
    def flocker(self) -> V1FlockerVolumeSource: ...
    @flocker.setter
    def flocker(self, flocker: V1FlockerVolumeSource) -> None: ...
    @property
    def gce_persistent_disk(self) -> V1GCEPersistentDiskVolumeSource: ...
    @gce_persistent_disk.setter
    def gce_persistent_disk(self, gce_persistent_disk: V1GCEPersistentDiskVolumeSource) -> None: ...
    @property
    def git_repo(self) -> V1GitRepoVolumeSource: ...
    @git_repo.setter
    def git_repo(self, git_repo: V1GitRepoVolumeSource) -> None: ...
    @property
    def glusterfs(self) -> V1GlusterfsVolumeSource: ...
    @glusterfs.setter
    def glusterfs(self, glusterfs: V1GlusterfsVolumeSource) -> None: ...
    @property
    def host_path(self) -> V1HostPathVolumeSource: ...
    @host_path.setter
    def host_path(self, host_path: V1HostPathVolumeSource) -> None: ...
    @property
    def image(self) -> V1ImageVolumeSource: ...
    @image.setter
    def image(self, image: V1ImageVolumeSource) -> None: ...
    @property
    def iscsi(self) -> V1ISCSIVolumeSource: ...
    @iscsi.setter
    def iscsi(self, iscsi: V1ISCSIVolumeSource) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def nfs(self) -> V1NFSVolumeSource: ...
    @nfs.setter
    def nfs(self, nfs: V1NFSVolumeSource) -> None: ...
    @property
    def persistent_volume_claim(self) -> V1PersistentVolumeClaimVolumeSource: ...
    @persistent_volume_claim.setter
    def persistent_volume_claim(self, persistent_volume_claim: V1PersistentVolumeClaimVolumeSource) -> None: ...
    @property
    def photon_persistent_disk(self) -> V1PhotonPersistentDiskVolumeSource: ...
    @photon_persistent_disk.setter
    def photon_persistent_disk(self, photon_persistent_disk: V1PhotonPersistentDiskVolumeSource) -> None: ...
    @property
    def portworx_volume(self) -> V1PortworxVolumeSource: ...
    @portworx_volume.setter
    def portworx_volume(self, portworx_volume: V1PortworxVolumeSource) -> None: ...
    @property
    def projected(self) -> V1ProjectedVolumeSource: ...
    @projected.setter
    def projected(self, projected: V1ProjectedVolumeSource) -> None: ...
    @property
    def quobyte(self) -> V1QuobyteVolumeSource: ...
    @quobyte.setter
    def quobyte(self, quobyte: V1QuobyteVolumeSource) -> None: ...
    @property
    def rbd(self) -> V1RBDVolumeSource: ...
    @rbd.setter
    def rbd(self, rbd: V1RBDVolumeSource) -> None: ...
    @property
    def scale_io(self) -> V1ScaleIOVolumeSource: ...
    @scale_io.setter
    def scale_io(self, scale_io: V1ScaleIOVolumeSource) -> None: ...
    @property
    def secret(self) -> V1SecretVolumeSource: ...
    @secret.setter
    def secret(self, secret: V1SecretVolumeSource) -> None: ...
    @property
    def storageos(self) -> V1StorageOSVolumeSource: ...
    @storageos.setter
    def storageos(self, storageos: V1StorageOSVolumeSource) -> None: ...
    @property
    def vsphere_volume(self) -> V1VsphereVirtualDiskVolumeSource: ...
    @vsphere_volume.setter
    def vsphere_volume(self, vsphere_volume: V1VsphereVirtualDiskVolumeSource) -> None: ...



from kubernetes_asyncio.client.models import V1NodeSelectorTerm
from typing import Any

class V1PreferredSchedulingTerm:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, preference: V1NodeSelectorTerm | None = None, weight: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def preference(self) -> V1NodeSelectorTerm: ...
    @preference.setter
    def preference(self, preference: V1NodeSelectorTerm) -> None: ...
    @property
    def weight(self) -> int: ...
    @weight.setter
    def weight(self, weight: int) -> None: ...



from kubernetes_asyncio.client.models import V1alpha1GroupVersionResource
from typing import Any

class V1alpha1StorageVersionMigrationSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, continue_token: str | None = None, resource: V1alpha1GroupVersionResource | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def continue_token(self) -> str: ...
    @continue_token.setter
    def continue_token(self, continue_token: str) -> None: ...
    @property
    def resource(self) -> V1alpha1GroupVersionResource: ...
    @resource.setter
    def resource(self, resource: V1alpha1GroupVersionResource) -> None: ...



from kubernetes_asyncio.client.models import ApiextensionsV1WebhookClientConfig
from typing import Any

class V1WebhookConversion:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, client_config: ApiextensionsV1WebhookClientConfig | None = None, conversion_review_versions: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def client_config(self) -> ApiextensionsV1WebhookClientConfig: ...
    @client_config.setter
    def client_config(self, client_config: ApiextensionsV1WebhookClientConfig) -> None: ...
    @property
    def conversion_review_versions(self) -> list[str]: ...
    @conversion_review_versions.setter
    def conversion_review_versions(self, conversion_review_versions: list[str]) -> None: ...



from kubernetes_asyncio.client.models import V1ClusterTrustBundleProjection
from kubernetes_asyncio.client.models import V1ConfigMapProjection
from kubernetes_asyncio.client.models import V1DownwardAPIProjection
from kubernetes_asyncio.client.models import V1PodCertificateProjection
from kubernetes_asyncio.client.models import V1SecretProjection
from kubernetes_asyncio.client.models import V1ServiceAccountTokenProjection
from typing import Any

class V1VolumeProjection:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, cluster_trust_bundle: V1ClusterTrustBundleProjection | None = None, config_map: V1ConfigMapProjection | None = None, downward_api: V1DownwardAPIProjection | None = None, pod_certificate: V1PodCertificateProjection | None = None, secret: V1SecretProjection | None = None, service_account_token: V1ServiceAccountTokenProjection | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def cluster_trust_bundle(self) -> V1ClusterTrustBundleProjection: ...
    @cluster_trust_bundle.setter
    def cluster_trust_bundle(self, cluster_trust_bundle: V1ClusterTrustBundleProjection) -> None: ...
    @property
    def config_map(self) -> V1ConfigMapProjection: ...
    @config_map.setter
    def config_map(self, config_map: V1ConfigMapProjection) -> None: ...
    @property
    def downward_api(self) -> V1DownwardAPIProjection: ...
    @downward_api.setter
    def downward_api(self, downward_api: V1DownwardAPIProjection) -> None: ...
    @property
    def pod_certificate(self) -> V1PodCertificateProjection: ...
    @pod_certificate.setter
    def pod_certificate(self, pod_certificate: V1PodCertificateProjection) -> None: ...
    @property
    def secret(self) -> V1SecretProjection: ...
    @secret.setter
    def secret(self, secret: V1SecretProjection) -> None: ...
    @property
    def service_account_token(self) -> V1ServiceAccountTokenProjection: ...
    @service_account_token.setter
    def service_account_token(self, service_account_token: V1ServiceAccountTokenProjection) -> None: ...



from datetime import datetime
from typing import Any

class V1StatefulSetCondition:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, last_transition_time: datetime | None = None, message: str | None = None, reason: str | None = None, status: str | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def last_transition_time(self) -> datetime: ...
    @last_transition_time.setter
    def last_transition_time(self, last_transition_time: datetime) -> None: ...
    @property
    def message(self) -> str: ...
    @message.setter
    def message(self, message: str) -> None: ...
    @property
    def reason(self) -> str: ...
    @reason.setter
    def reason(self, reason: str) -> None: ...
    @property
    def status(self) -> str: ...
    @status.setter
    def status(self, status: str) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from typing import Any

class V1VolumeResourceRequirements:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, limits: dict[str, str] | None = None, requests: dict[str, str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def limits(self) -> dict[str, str]: ...
    @limits.setter
    def limits(self, limits: dict[str, str]) -> None: ...
    @property
    def requests(self) -> dict[str, str]: ...
    @requests.setter
    def requests(self, requests: dict[str, str]) -> None: ...



from kubernetes_asyncio.client.models import V1NamespaceCondition
from typing import Any

class V1NamespaceStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, conditions: list[V1NamespaceCondition] | None = None, phase: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def conditions(self) -> list[V1NamespaceCondition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1NamespaceCondition]) -> None: ...
    @property
    def phase(self) -> str: ...
    @phase.setter
    def phase(self, phase: str) -> None: ...



from typing import Any

class V1ValidationRule:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, field_path: str | None = None, message: str | None = None, message_expression: str | None = None, optional_old_self: bool | None = None, reason: str | None = None, rule: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def field_path(self) -> str: ...
    @field_path.setter
    def field_path(self, field_path: str) -> None: ...
    @property
    def message(self) -> str: ...
    @message.setter
    def message(self, message: str) -> None: ...
    @property
    def message_expression(self) -> str: ...
    @message_expression.setter
    def message_expression(self, message_expression: str) -> None: ...
    @property
    def optional_old_self(self) -> bool: ...
    @optional_old_self.setter
    def optional_old_self(self, optional_old_self: bool) -> None: ...
    @property
    def reason(self) -> str: ...
    @reason.setter
    def reason(self, reason: str) -> None: ...
    @property
    def rule(self) -> str: ...
    @rule.setter
    def rule(self, rule: str) -> None: ...



from kubernetes_asyncio.client.models import V1alpha3DeviceTaint
from kubernetes_asyncio.client.models import V1alpha3DeviceTaintSelector
from typing import Any

class V1alpha3DeviceTaintRuleSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, device_selector: V1alpha3DeviceTaintSelector | None = None, taint: V1alpha3DeviceTaint | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def device_selector(self) -> V1alpha3DeviceTaintSelector: ...
    @device_selector.setter
    def device_selector(self, device_selector: V1alpha3DeviceTaintSelector) -> None: ...
    @property
    def taint(self) -> V1alpha3DeviceTaint: ...
    @taint.setter
    def taint(self, taint: V1alpha3DeviceTaint) -> None: ...



from kubernetes_asyncio.client.models import V1LabelSelector
from typing import Any

class V1alpha1ParamRef:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, name: str | None = None, namespace: str | None = None, parameter_not_found_action: str | None = None, selector: V1LabelSelector | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def namespace(self) -> str: ...
    @namespace.setter
    def namespace(self, namespace: str) -> None: ...
    @property
    def parameter_not_found_action(self) -> str: ...
    @parameter_not_found_action.setter
    def parameter_not_found_action(self, parameter_not_found_action: str) -> None: ...
    @property
    def selector(self) -> V1LabelSelector: ...
    @selector.setter
    def selector(self, selector: V1LabelSelector) -> None: ...



from kubernetes_asyncio.client.models import V1FlowDistinguisherMethod
from kubernetes_asyncio.client.models import V1PolicyRulesWithSubjects
from kubernetes_asyncio.client.models import V1PriorityLevelConfigurationReference
from typing import Any

class V1FlowSchemaSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, distinguisher_method: V1FlowDistinguisherMethod | None = None, matching_precedence: int | None = None, priority_level_configuration: V1PriorityLevelConfigurationReference | None = None, rules: list[V1PolicyRulesWithSubjects] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def distinguisher_method(self) -> V1FlowDistinguisherMethod: ...
    @distinguisher_method.setter
    def distinguisher_method(self, distinguisher_method: V1FlowDistinguisherMethod) -> None: ...
    @property
    def matching_precedence(self) -> int: ...
    @matching_precedence.setter
    def matching_precedence(self, matching_precedence: int) -> None: ...
    @property
    def priority_level_configuration(self) -> V1PriorityLevelConfigurationReference: ...
    @priority_level_configuration.setter
    def priority_level_configuration(self, priority_level_configuration: V1PriorityLevelConfigurationReference) -> None: ...
    @property
    def rules(self) -> list[V1PolicyRulesWithSubjects]: ...
    @rules.setter
    def rules(self, rules: list[V1PolicyRulesWithSubjects]) -> None: ...



from kubernetes_asyncio.client.models import V1AttachedVolume
from kubernetes_asyncio.client.models import V1ContainerImage
from kubernetes_asyncio.client.models import V1NodeAddress
from kubernetes_asyncio.client.models import V1NodeCondition
from kubernetes_asyncio.client.models import V1NodeConfigStatus
from kubernetes_asyncio.client.models import V1NodeDaemonEndpoints
from kubernetes_asyncio.client.models import V1NodeFeatures
from kubernetes_asyncio.client.models import V1NodeRuntimeHandler
from kubernetes_asyncio.client.models import V1NodeSystemInfo
from typing import Any

class V1NodeStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, addresses: list[V1NodeAddress] | None = None, allocatable: dict[str, str] | None = None, capacity: dict[str, str] | None = None, conditions: list[V1NodeCondition] | None = None, config: V1NodeConfigStatus | None = None, daemon_endpoints: V1NodeDaemonEndpoints | None = None, features: V1NodeFeatures | None = None, images: list[V1ContainerImage] | None = None, node_info: V1NodeSystemInfo | None = None, phase: str | None = None, runtime_handlers: list[V1NodeRuntimeHandler] | None = None, volumes_attached: list[V1AttachedVolume] | None = None, volumes_in_use: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def addresses(self) -> list[V1NodeAddress]: ...
    @addresses.setter
    def addresses(self, addresses: list[V1NodeAddress]) -> None: ...
    @property
    def allocatable(self) -> dict[str, str]: ...
    @allocatable.setter
    def allocatable(self, allocatable: dict[str, str]) -> None: ...
    @property
    def capacity(self) -> dict[str, str]: ...
    @capacity.setter
    def capacity(self, capacity: dict[str, str]) -> None: ...
    @property
    def conditions(self) -> list[V1NodeCondition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1NodeCondition]) -> None: ...
    @property
    def config(self) -> V1NodeConfigStatus: ...
    @config.setter
    def config(self, config: V1NodeConfigStatus) -> None: ...
    @property
    def daemon_endpoints(self) -> V1NodeDaemonEndpoints: ...
    @daemon_endpoints.setter
    def daemon_endpoints(self, daemon_endpoints: V1NodeDaemonEndpoints) -> None: ...
    @property
    def features(self) -> V1NodeFeatures: ...
    @features.setter
    def features(self, features: V1NodeFeatures) -> None: ...
    @property
    def images(self) -> list[V1ContainerImage]: ...
    @images.setter
    def images(self, images: list[V1ContainerImage]) -> None: ...
    @property
    def node_info(self) -> V1NodeSystemInfo: ...
    @node_info.setter
    def node_info(self, node_info: V1NodeSystemInfo) -> None: ...
    @property
    def phase(self) -> str: ...
    @phase.setter
    def phase(self, phase: str) -> None: ...
    @property
    def runtime_handlers(self) -> list[V1NodeRuntimeHandler]: ...
    @runtime_handlers.setter
    def runtime_handlers(self, runtime_handlers: list[V1NodeRuntimeHandler]) -> None: ...
    @property
    def volumes_attached(self) -> list[V1AttachedVolume]: ...
    @volumes_attached.setter
    def volumes_attached(self, volumes_attached: list[V1AttachedVolume]) -> None: ...
    @property
    def volumes_in_use(self) -> list[str]: ...
    @volumes_in_use.setter
    def volumes_in_use(self, volumes_in_use: list[str]) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1ClusterRole
from kubernetes_asyncio.client.models import V1ClusterRoleBinding
from kubernetes_asyncio.client.models import V1ClusterRoleBindingList
from kubernetes_asyncio.client.models import V1ClusterRoleList
from kubernetes_asyncio.client.models import V1Role
from kubernetes_asyncio.client.models import V1RoleBinding
from kubernetes_asyncio.client.models import V1RoleBindingList
from kubernetes_asyncio.client.models import V1RoleList
from kubernetes_asyncio.client.models import V1Status
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class RbacAuthorizationV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_cluster_role(self, body: V1ClusterRole, **kwargs: Any) -> Awaitable[V1ClusterRole]: ...
    def create_cluster_role_binding(self, body: V1ClusterRoleBinding, **kwargs: Any) -> Awaitable[V1ClusterRoleBinding]: ...
    def create_cluster_role_binding_with_http_info(self, body: V1ClusterRoleBinding, **kwargs: Any) -> Awaitable[tuple[V1ClusterRoleBinding, int, CIMultiDictProxy]]: ...
    def create_cluster_role_with_http_info(self, body: V1ClusterRole, **kwargs: Any) -> Awaitable[tuple[V1ClusterRole, int, CIMultiDictProxy]]: ...
    def create_namespaced_role(self, namespace: str, body: V1Role, **kwargs: Any) -> Awaitable[V1Role]: ...
    def create_namespaced_role_binding(self, namespace: str, body: V1RoleBinding, **kwargs: Any) -> Awaitable[V1RoleBinding]: ...
    def create_namespaced_role_binding_with_http_info(self, namespace: str, body: V1RoleBinding, **kwargs: Any) -> Awaitable[tuple[V1RoleBinding, int, CIMultiDictProxy]]: ...
    def create_namespaced_role_with_http_info(self, namespace: str, body: V1Role, **kwargs: Any) -> Awaitable[tuple[V1Role, int, CIMultiDictProxy]]: ...
    def delete_cluster_role(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_cluster_role_binding(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_cluster_role_binding_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_cluster_role_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_cluster_role(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_cluster_role_binding(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_cluster_role_binding_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_cluster_role_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_role(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_role_binding(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_role_binding_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_role_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_role(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_role_binding(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_role_binding_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_role_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_cluster_role(self, **kwargs: Any) -> Awaitable[V1ClusterRoleList]: ...
    def list_cluster_role_binding(self, **kwargs: Any) -> Awaitable[V1ClusterRoleBindingList]: ...
    def list_cluster_role_binding_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1ClusterRoleBindingList, int, CIMultiDictProxy]]: ...
    def list_cluster_role_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1ClusterRoleList, int, CIMultiDictProxy]]: ...
    def list_namespaced_role(self, namespace: str, **kwargs: Any) -> Awaitable[V1RoleList]: ...
    def list_namespaced_role_binding(self, namespace: str, **kwargs: Any) -> Awaitable[V1RoleBindingList]: ...
    def list_namespaced_role_binding_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1RoleBindingList, int, CIMultiDictProxy]]: ...
    def list_namespaced_role_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1RoleList, int, CIMultiDictProxy]]: ...
    def list_role_binding_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1RoleBindingList]: ...
    def list_role_binding_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1RoleBindingList, int, CIMultiDictProxy]]: ...
    def list_role_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1RoleList]: ...
    def list_role_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1RoleList, int, CIMultiDictProxy]]: ...
    def patch_cluster_role(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1ClusterRole]: ...
    def patch_cluster_role_binding(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1ClusterRoleBinding]: ...
    def patch_cluster_role_binding_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ClusterRoleBinding, int, CIMultiDictProxy]]: ...
    def patch_cluster_role_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ClusterRole, int, CIMultiDictProxy]]: ...
    def patch_namespaced_role(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1Role]: ...
    def patch_namespaced_role_binding(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1RoleBinding]: ...
    def patch_namespaced_role_binding_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1RoleBinding, int, CIMultiDictProxy]]: ...
    def patch_namespaced_role_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1Role, int, CIMultiDictProxy]]: ...
    def read_cluster_role(self, name: str, **kwargs: Any) -> Awaitable[V1ClusterRole]: ...
    def read_cluster_role_binding(self, name: str, **kwargs: Any) -> Awaitable[V1ClusterRoleBinding]: ...
    def read_cluster_role_binding_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1ClusterRoleBinding, int, CIMultiDictProxy]]: ...
    def read_cluster_role_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1ClusterRole, int, CIMultiDictProxy]]: ...
    def read_namespaced_role(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Role]: ...
    def read_namespaced_role_binding(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1RoleBinding]: ...
    def read_namespaced_role_binding_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1RoleBinding, int, CIMultiDictProxy]]: ...
    def read_namespaced_role_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Role, int, CIMultiDictProxy]]: ...
    def replace_cluster_role(self, name: str, body: V1ClusterRole, **kwargs: Any) -> Awaitable[V1ClusterRole]: ...
    def replace_cluster_role_binding(self, name: str, body: V1ClusterRoleBinding, **kwargs: Any) -> Awaitable[V1ClusterRoleBinding]: ...
    def replace_cluster_role_binding_with_http_info(self, name: str, body: V1ClusterRoleBinding, **kwargs: Any) -> Awaitable[tuple[V1ClusterRoleBinding, int, CIMultiDictProxy]]: ...
    def replace_cluster_role_with_http_info(self, name: str, body: V1ClusterRole, **kwargs: Any) -> Awaitable[tuple[V1ClusterRole, int, CIMultiDictProxy]]: ...
    def replace_namespaced_role(self, name: str, namespace: str, body: V1Role, **kwargs: Any) -> Awaitable[V1Role]: ...
    def replace_namespaced_role_binding(self, name: str, namespace: str, body: V1RoleBinding, **kwargs: Any) -> Awaitable[V1RoleBinding]: ...
    def replace_namespaced_role_binding_with_http_info(self, name: str, namespace: str, body: V1RoleBinding, **kwargs: Any) -> Awaitable[tuple[V1RoleBinding, int, CIMultiDictProxy]]: ...
    def replace_namespaced_role_with_http_info(self, name: str, namespace: str, body: V1Role, **kwargs: Any) -> Awaitable[tuple[V1Role, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1LabelSelectorRequirement
from typing import Any

class V1LabelSelector:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, match_expressions: list[V1LabelSelectorRequirement] | None = None, match_labels: dict[str, str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def match_expressions(self) -> list[V1LabelSelectorRequirement]: ...
    @match_expressions.setter
    def match_expressions(self, match_expressions: list[V1LabelSelectorRequirement]) -> None: ...
    @property
    def match_labels(self) -> dict[str, str]: ...
    @match_labels.setter
    def match_labels(self, match_labels: dict[str, str]) -> None: ...



from kubernetes_asyncio.client.models import V1IngressClassParametersReference
from typing import Any

class V1IngressClassSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, controller: str | None = None, parameters: V1IngressClassParametersReference | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def controller(self) -> str: ...
    @controller.setter
    def controller(self, controller: str) -> None: ...
    @property
    def parameters(self) -> V1IngressClassParametersReference: ...
    @parameters.setter
    def parameters(self, parameters: V1IngressClassParametersReference) -> None: ...



from typing import Any

class V1WindowsSecurityContextOptions:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, gmsa_credential_spec: str | None = None, gmsa_credential_spec_name: str | None = None, host_process: bool | None = None, run_as_user_name: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def gmsa_credential_spec(self) -> str: ...
    @gmsa_credential_spec.setter
    def gmsa_credential_spec(self, gmsa_credential_spec: str) -> None: ...
    @property
    def gmsa_credential_spec_name(self) -> str: ...
    @gmsa_credential_spec_name.setter
    def gmsa_credential_spec_name(self, gmsa_credential_spec_name: str) -> None: ...
    @property
    def host_process(self) -> bool: ...
    @host_process.setter
    def host_process(self, host_process: bool) -> None: ...
    @property
    def run_as_user_name(self) -> str: ...
    @run_as_user_name.setter
    def run_as_user_name(self, run_as_user_name: str) -> None: ...



from typing import Any

class VersionInfo:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, build_date: str | None = None, compiler: str | None = None, emulation_major: str | None = None, emulation_minor: str | None = None, git_commit: str | None = None, git_tree_state: str | None = None, git_version: str | None = None, go_version: str | None = None, major: str | None = None, min_compatibility_major: str | None = None, min_compatibility_minor: str | None = None, minor: str | None = None, platform: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def build_date(self) -> str: ...
    @build_date.setter
    def build_date(self, build_date: str) -> None: ...
    @property
    def compiler(self) -> str: ...
    @compiler.setter
    def compiler(self, compiler: str) -> None: ...
    @property
    def emulation_major(self) -> str: ...
    @emulation_major.setter
    def emulation_major(self, emulation_major: str) -> None: ...
    @property
    def emulation_minor(self) -> str: ...
    @emulation_minor.setter
    def emulation_minor(self, emulation_minor: str) -> None: ...
    @property
    def git_commit(self) -> str: ...
    @git_commit.setter
    def git_commit(self, git_commit: str) -> None: ...
    @property
    def git_tree_state(self) -> str: ...
    @git_tree_state.setter
    def git_tree_state(self, git_tree_state: str) -> None: ...
    @property
    def git_version(self) -> str: ...
    @git_version.setter
    def git_version(self, git_version: str) -> None: ...
    @property
    def go_version(self) -> str: ...
    @go_version.setter
    def go_version(self, go_version: str) -> None: ...
    @property
    def major(self) -> str: ...
    @major.setter
    def major(self, major: str) -> None: ...
    @property
    def min_compatibility_major(self) -> str: ...
    @min_compatibility_major.setter
    def min_compatibility_major(self, min_compatibility_major: str) -> None: ...
    @property
    def min_compatibility_minor(self) -> str: ...
    @min_compatibility_minor.setter
    def min_compatibility_minor(self, min_compatibility_minor: str) -> None: ...
    @property
    def minor(self) -> str: ...
    @minor.setter
    def minor(self, minor: str) -> None: ...
    @property
    def platform(self) -> str: ...
    @platform.setter
    def platform(self, platform: str) -> None: ...



from datetime import datetime
from kubernetes_asyncio.client.models import CoreV1EventSeries
from kubernetes_asyncio.client.models import V1EventSource
from kubernetes_asyncio.client.models import V1ObjectMeta
from kubernetes_asyncio.client.models import V1ObjectReference
from typing import Any

class CoreV1Event:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, action: str | None = None, api_version: str | None = None, count: int | None = None, event_time: datetime | None = None, first_timestamp: datetime | None = None, involved_object: V1ObjectReference | None = None, kind: str | None = None, last_timestamp: datetime | None = None, message: str | None = None, metadata: V1ObjectMeta | None = None, reason: str | None = None, related: V1ObjectReference | None = None, reporting_component: str | None = None, reporting_instance: str | None = None, series: CoreV1EventSeries | None = None, source: V1EventSource | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def action(self) -> str: ...
    @action.setter
    def action(self, action: str) -> None: ...
    @property
    def api_version(self) -> str: ...
    @api_version.setter
    def api_version(self, api_version: str) -> None: ...
    @property
    def count(self) -> int: ...
    @count.setter
    def count(self, count: int) -> None: ...
    @property
    def event_time(self) -> datetime: ...
    @event_time.setter
    def event_time(self, event_time: datetime) -> None: ...
    @property
    def first_timestamp(self) -> datetime: ...
    @first_timestamp.setter
    def first_timestamp(self, first_timestamp: datetime) -> None: ...
    @property
    def involved_object(self) -> V1ObjectReference: ...
    @involved_object.setter
    def involved_object(self, involved_object: V1ObjectReference) -> None: ...
    @property
    def kind(self) -> str: ...
    @kind.setter
    def kind(self, kind: str) -> None: ...
    @property
    def last_timestamp(self) -> datetime: ...
    @last_timestamp.setter
    def last_timestamp(self, last_timestamp: datetime) -> None: ...
    @property
    def message(self) -> str: ...
    @message.setter
    def message(self, message: str) -> None: ...
    @property
    def metadata(self) -> V1ObjectMeta: ...
    @metadata.setter
    def metadata(self, metadata: V1ObjectMeta) -> None: ...
    @property
    def reason(self) -> str: ...
    @reason.setter
    def reason(self, reason: str) -> None: ...
    @property
    def related(self) -> V1ObjectReference: ...
    @related.setter
    def related(self, related: V1ObjectReference) -> None: ...
    @property
    def reporting_component(self) -> str: ...
    @reporting_component.setter
    def reporting_component(self, reporting_component: str) -> None: ...
    @property
    def reporting_instance(self) -> str: ...
    @reporting_instance.setter
    def reporting_instance(self, reporting_instance: str) -> None: ...
    @property
    def series(self) -> CoreV1EventSeries: ...
    @series.setter
    def series(self, series: CoreV1EventSeries) -> None: ...
    @property
    def source(self) -> V1EventSource: ...
    @source.setter
    def source(self, source: V1EventSource) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from datetime import datetime
from typing import Any

class V1VolumeError:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, error_code: int | None = None, message: str | None = None, time: datetime | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def error_code(self) -> int: ...
    @error_code.setter
    def error_code(self, error_code: int) -> None: ...
    @property
    def message(self) -> str: ...
    @message.setter
    def message(self, message: str) -> None: ...
    @property
    def time(self) -> datetime: ...
    @time.setter
    def time(self, time: datetime) -> None: ...



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.34.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1ExpressionWarning(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'field_ref': 'str',
        'warning': 'str'
    }

    attribute_map = {
        'field_ref': 'fieldRef',
        'warning': 'warning'
    }

    def __init__(self, field_ref=None, warning=None, local_vars_configuration=None):  # noqa: E501
        """V1ExpressionWarning - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._field_ref = None
        self._warning = None
        self.discriminator = None

        self.field_ref = field_ref
        self.warning = warning

    @property
    def field_ref(self):
        """Gets the field_ref of this V1ExpressionWarning.  # noqa: E501

        The path to the field that refers the expression. For example, the reference to the expression of the first item of validations is \"spec.validations[0].expression\"  # noqa: E501

        :return: The field_ref of this V1ExpressionWarning.  # noqa: E501
        :rtype: str
        """
        return self._field_ref

    @field_ref.setter
    def field_ref(self, field_ref):
        """Sets the field_ref of this V1ExpressionWarning.

        The path to the field that refers the expression. For example, the reference to the expression of the first item of validations is \"spec.validations[0].expression\"  # noqa: E501

        :param field_ref: The field_ref of this V1ExpressionWarning.  # noqa: E501
        :type field_ref: str
        """
        if self.local_vars_configuration.client_side_validation and field_ref is None:  # noqa: E501
            raise ValueError("Invalid value for `field_ref`, must not be `None`")  # noqa: E501

        self._field_ref = field_ref

    @property
    def warning(self):
        """Gets the warning of this V1ExpressionWarning.  # noqa: E501

        The content of type checking information in a human-readable form. Each line of the warning contains the type that the expression is checked against, followed by the type check error from the compiler.  # noqa: E501

        :return: The warning of this V1ExpressionWarning.  # noqa: E501
        :rtype: str
        """
        return self._warning

    @warning.setter
    def warning(self, warning):
        """Sets the warning of this V1ExpressionWarning.

        The content of type checking information in a human-readable form. Each line of the warning contains the type that the expression is checked against, followed by the type check error from the compiler.  # noqa: E501

        :param warning: The warning of this V1ExpressionWarning.  # noqa: E501
        :type warning: str
        """
        if self.local_vars_configuration.client_side_validation and warning is None:  # noqa: E501
            raise ValueError("Invalid value for `warning`, must not be `None`")  # noqa: E501

        self._warning = warning

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ExpressionWarning):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1ExpressionWarning):
            return True

        return self.to_dict() != other.to_dict()



from kubernetes_asyncio.client.models import V2HPAScalingRules
from typing import Any

class V2HorizontalPodAutoscalerBehavior:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, scale_down: V2HPAScalingRules | None = None, scale_up: V2HPAScalingRules | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def scale_down(self) -> V2HPAScalingRules: ...
    @scale_down.setter
    def scale_down(self, scale_down: V2HPAScalingRules) -> None: ...
    @property
    def scale_up(self) -> V2HPAScalingRules: ...
    @scale_up.setter
    def scale_up(self, scale_up: V2HPAScalingRules) -> None: ...



from typing import Any

class V1ResourcePolicyRule:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, api_groups: list[str] | None = None, cluster_scope: bool | None = None, namespaces: list[str] | None = None, resources: list[str] | None = None, verbs: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def api_groups(self) -> list[str]: ...
    @api_groups.setter
    def api_groups(self, api_groups: list[str]) -> None: ...
    @property
    def cluster_scope(self) -> bool: ...
    @cluster_scope.setter
    def cluster_scope(self, cluster_scope: bool) -> None: ...
    @property
    def namespaces(self) -> list[str]: ...
    @namespaces.setter
    def namespaces(self, namespaces: list[str]) -> None: ...
    @property
    def resources(self) -> list[str]: ...
    @resources.setter
    def resources(self, resources: list[str]) -> None: ...
    @property
    def verbs(self) -> list[str]: ...
    @verbs.setter
    def verbs(self, verbs: list[str]) -> None: ...



from typing import Any

class V1GitRepoVolumeSource:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, directory: str | None = None, repository: str | None = None, revision: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def directory(self) -> str: ...
    @directory.setter
    def directory(self, directory: str) -> None: ...
    @property
    def repository(self) -> str: ...
    @repository.setter
    def repository(self, repository: str) -> None: ...
    @property
    def revision(self) -> str: ...
    @revision.setter
    def revision(self, revision: str) -> None: ...



from kubernetes_asyncio.client.models import V1PodAffinityTerm
from typing import Any

class V1WeightedPodAffinityTerm:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, pod_affinity_term: V1PodAffinityTerm | None = None, weight: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def pod_affinity_term(self) -> V1PodAffinityTerm: ...
    @pod_affinity_term.setter
    def pod_affinity_term(self, pod_affinity_term: V1PodAffinityTerm) -> None: ...
    @property
    def weight(self) -> int: ...
    @weight.setter
    def weight(self, weight: int) -> None: ...



from kubernetes_asyncio.client.models import V1beta2CapacityRequestPolicyRange
from typing import Any

class V1beta2CapacityRequestPolicy:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, default: str | None = None, valid_range: V1beta2CapacityRequestPolicyRange | None = None, valid_values: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def default(self) -> str: ...
    @default.setter
    def default(self, default: str) -> None: ...
    @property
    def valid_range(self) -> V1beta2CapacityRequestPolicyRange: ...
    @valid_range.setter
    def valid_range(self, valid_range: V1beta2CapacityRequestPolicyRange) -> None: ...
    @property
    def valid_values(self) -> list[str]: ...
    @valid_values.setter
    def valid_values(self, valid_values: list[str]) -> None: ...



from kubernetes_asyncio.client.models import V1DaemonEndpoint
from typing import Any

class V1NodeDaemonEndpoints:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, kubelet_endpoint: V1DaemonEndpoint | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def kubelet_endpoint(self) -> V1DaemonEndpoint: ...
    @kubelet_endpoint.setter
    def kubelet_endpoint(self, kubelet_endpoint: V1DaemonEndpoint) -> None: ...



from typing import Any

class V1alpha1GroupVersionResource:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, group: str | None = None, resource: str | None = None, version: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def group(self) -> str: ...
    @group.setter
    def group(self, group: str) -> None: ...
    @property
    def resource(self) -> str: ...
    @resource.setter
    def resource(self, resource: str) -> None: ...
    @property
    def version(self) -> str: ...
    @version.setter
    def version(self, version: str) -> None: ...



from typing import Any

class V1beta1DeviceToleration:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, effect: str | None = None, key: str | None = None, operator: str | None = None, toleration_seconds: int | None = None, value: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def effect(self) -> str: ...
    @effect.setter
    def effect(self, effect: str) -> None: ...
    @property
    def key(self) -> str: ...
    @key.setter
    def key(self, key: str) -> None: ...
    @property
    def operator(self) -> str: ...
    @operator.setter
    def operator(self, operator: str) -> None: ...
    @property
    def toleration_seconds(self) -> int: ...
    @toleration_seconds.setter
    def toleration_seconds(self, toleration_seconds: int) -> None: ...
    @property
    def value(self) -> str: ...
    @value.setter
    def value(self, value: str) -> None: ...



from kubernetes_asyncio.client.models import V1ExternalDocumentation
from kubernetes_asyncio.client.models import V1ValidationRule
from typing import Any

class V1JSONSchemaProps:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, ref: str | None = None, schema: str | None = None, additional_items: Any | None = None, additional_properties: Any | None = None, all_of: list[V1JSONSchemaProps] | None = None, any_of: list[V1JSONSchemaProps] | None = None, default: Any | None = None, definitions: dict[str, V1JSONSchemaProps] | None = None, dependencies: dict[str, Any] | None = None, description: str | None = None, enum: list[Any] | None = None, example: Any | None = None, exclusive_maximum: bool | None = None, exclusive_minimum: bool | None = None, external_docs: V1ExternalDocumentation | None = None, format: str | None = None, id: str | None = None, items: Any | None = None, max_items: int | None = None, max_length: int | None = None, max_properties: int | None = None, maximum: float | None = None, min_items: int | None = None, min_length: int | None = None, min_properties: int | None = None, minimum: float | None = None, multiple_of: float | None = None, _not: V1JSONSchemaProps | None = None, nullable: bool | None = None, one_of: list[V1JSONSchemaProps] | None = None, pattern: str | None = None, pattern_properties: dict[str, V1JSONSchemaProps] | None = None, properties: dict[str, V1JSONSchemaProps] | None = None, required: list[str] | None = None, title: str | None = None, type: str | None = None, unique_items: bool | None = None, x_kubernetes_embedded_resource: bool | None = None, x_kubernetes_int_or_string: bool | None = None, x_kubernetes_list_map_keys: list[str] | None = None, x_kubernetes_list_type: str | None = None, x_kubernetes_map_type: str | None = None, x_kubernetes_preserve_unknown_fields: bool | None = None, x_kubernetes_validations: list[V1ValidationRule] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def _not(self) -> V1JSONSchemaProps: ...
    @_not.setter
    def _not(self, _not: V1JSONSchemaProps) -> None: ...
    @property
    def additional_items(self) -> Any: ...
    @additional_items.setter
    def additional_items(self, additional_items: Any) -> None: ...
    @property
    def additional_properties(self) -> Any: ...
    @additional_properties.setter
    def additional_properties(self, additional_properties: Any) -> None: ...
    @property
    def all_of(self) -> list[V1JSONSchemaProps]: ...
    @all_of.setter
    def all_of(self, all_of: list[V1JSONSchemaProps]) -> None: ...
    @property
    def any_of(self) -> list[V1JSONSchemaProps]: ...
    @any_of.setter
    def any_of(self, any_of: list[V1JSONSchemaProps]) -> None: ...
    @property
    def default(self) -> Any: ...
    @default.setter
    def default(self, default: Any) -> None: ...
    @property
    def definitions(self) -> dict[str, V1JSONSchemaProps]: ...
    @definitions.setter
    def definitions(self, definitions: dict[str, V1JSONSchemaProps]) -> None: ...
    @property
    def dependencies(self) -> dict[str, Any]: ...
    @dependencies.setter
    def dependencies(self, dependencies: dict[str, Any]) -> None: ...
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, description: str) -> None: ...
    @property
    def enum(self) -> list[Any]: ...
    @enum.setter
    def enum(self, enum: list[Any]) -> None: ...
    @property
    def example(self) -> Any: ...
    @example.setter
    def example(self, example: Any) -> None: ...
    @property
    def exclusive_maximum(self) -> bool: ...
    @exclusive_maximum.setter
    def exclusive_maximum(self, exclusive_maximum: bool) -> None: ...
    @property
    def exclusive_minimum(self) -> bool: ...
    @exclusive_minimum.setter
    def exclusive_minimum(self, exclusive_minimum: bool) -> None: ...
    @property
    def external_docs(self) -> V1ExternalDocumentation: ...
    @external_docs.setter
    def external_docs(self, external_docs: V1ExternalDocumentation) -> None: ...
    @property
    def format(self) -> str: ...
    @format.setter
    def format(self, format: str) -> None: ...
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, id: str) -> None: ...
    @property
    def items(self) -> Any: ...
    @items.setter
    def items(self, items: Any) -> None: ...
    @property
    def max_items(self) -> int: ...
    @max_items.setter
    def max_items(self, max_items: int) -> None: ...
    @property
    def max_length(self) -> int: ...
    @max_length.setter
    def max_length(self, max_length: int) -> None: ...
    @property
    def max_properties(self) -> int: ...
    @max_properties.setter
    def max_properties(self, max_properties: int) -> None: ...
    @property
    def maximum(self) -> float: ...
    @maximum.setter
    def maximum(self, maximum: float) -> None: ...
    @property
    def min_items(self) -> int: ...
    @min_items.setter
    def min_items(self, min_items: int) -> None: ...
    @property
    def min_length(self) -> int: ...
    @min_length.setter
    def min_length(self, min_length: int) -> None: ...
    @property
    def min_properties(self) -> int: ...
    @min_properties.setter
    def min_properties(self, min_properties: int) -> None: ...
    @property
    def minimum(self) -> float: ...
    @minimum.setter
    def minimum(self, minimum: float) -> None: ...
    @property
    def multiple_of(self) -> float: ...
    @multiple_of.setter
    def multiple_of(self, multiple_of: float) -> None: ...
    @property
    def nullable(self) -> bool: ...
    @nullable.setter
    def nullable(self, nullable: bool) -> None: ...
    @property
    def one_of(self) -> list[V1JSONSchemaProps]: ...
    @one_of.setter
    def one_of(self, one_of: list[V1JSONSchemaProps]) -> None: ...
    @property
    def pattern(self) -> str: ...
    @pattern.setter
    def pattern(self, pattern: str) -> None: ...
    @property
    def pattern_properties(self) -> dict[str, V1JSONSchemaProps]: ...
    @pattern_properties.setter
    def pattern_properties(self, pattern_properties: dict[str, V1JSONSchemaProps]) -> None: ...
    @property
    def properties(self) -> dict[str, V1JSONSchemaProps]: ...
    @properties.setter
    def properties(self, properties: dict[str, V1JSONSchemaProps]) -> None: ...
    @property
    def ref(self) -> str: ...
    @ref.setter
    def ref(self, ref: str) -> None: ...
    @property
    def required(self) -> list[str]: ...
    @required.setter
    def required(self, required: list[str]) -> None: ...
    @property
    def schema(self) -> str: ...
    @schema.setter
    def schema(self, schema: str) -> None: ...
    @property
    def title(self) -> str: ...
    @title.setter
    def title(self, title: str) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...
    @property
    def unique_items(self) -> bool: ...
    @unique_items.setter
    def unique_items(self, unique_items: bool) -> None: ...
    @property
    def x_kubernetes_embedded_resource(self) -> bool: ...
    @x_kubernetes_embedded_resource.setter
    def x_kubernetes_embedded_resource(self, x_kubernetes_embedded_resource: bool) -> None: ...
    @property
    def x_kubernetes_int_or_string(self) -> bool: ...
    @x_kubernetes_int_or_string.setter
    def x_kubernetes_int_or_string(self, x_kubernetes_int_or_string: bool) -> None: ...
    @property
    def x_kubernetes_list_map_keys(self) -> list[str]: ...
    @x_kubernetes_list_map_keys.setter
    def x_kubernetes_list_map_keys(self, x_kubernetes_list_map_keys: list[str]) -> None: ...
    @property
    def x_kubernetes_list_type(self) -> str: ...
    @x_kubernetes_list_type.setter
    def x_kubernetes_list_type(self, x_kubernetes_list_type: str) -> None: ...
    @property
    def x_kubernetes_map_type(self) -> str: ...
    @x_kubernetes_map_type.setter
    def x_kubernetes_map_type(self, x_kubernetes_map_type: str) -> None: ...
    @property
    def x_kubernetes_preserve_unknown_fields(self) -> bool: ...
    @x_kubernetes_preserve_unknown_fields.setter
    def x_kubernetes_preserve_unknown_fields(self, x_kubernetes_preserve_unknown_fields: bool) -> None: ...
    @property
    def x_kubernetes_validations(self) -> list[V1ValidationRule]: ...
    @x_kubernetes_validations.setter
    def x_kubernetes_validations(self, x_kubernetes_validations: list[V1ValidationRule]) -> None: ...



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.34.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1NodeAffinity(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'preferred_during_scheduling_ignored_during_execution': 'list[V1PreferredSchedulingTerm]',
        'required_during_scheduling_ignored_during_execution': 'V1NodeSelector'
    }

    attribute_map = {
        'preferred_during_scheduling_ignored_during_execution': 'preferredDuringSchedulingIgnoredDuringExecution',
        'required_during_scheduling_ignored_during_execution': 'requiredDuringSchedulingIgnoredDuringExecution'
    }

    def __init__(self, preferred_during_scheduling_ignored_during_execution=None, required_during_scheduling_ignored_during_execution=None, local_vars_configuration=None):  # noqa: E501
        """V1NodeAffinity - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._preferred_during_scheduling_ignored_during_execution = None
        self._required_during_scheduling_ignored_during_execution = None
        self.discriminator = None

        if preferred_during_scheduling_ignored_during_execution is not None:
            self.preferred_during_scheduling_ignored_during_execution = preferred_during_scheduling_ignored_during_execution
        if required_during_scheduling_ignored_during_execution is not None:
            self.required_during_scheduling_ignored_during_execution = required_during_scheduling_ignored_during_execution

    @property
    def preferred_during_scheduling_ignored_during_execution(self):
        """Gets the preferred_during_scheduling_ignored_during_execution of this V1NodeAffinity.  # noqa: E501

        The scheduler will prefer to schedule pods to nodes that satisfy the affinity expressions specified by this field, but it may choose a node that violates one or more of the expressions. The node that is most preferred is the one with the greatest sum of weights, i.e. for each node that meets all of the scheduling requirements (resource request, requiredDuringScheduling affinity expressions, etc.), compute a sum by iterating through the elements of this field and adding \"weight\" to the sum if the node matches the corresponding matchExpressions; the node(s) with the highest sum are the most preferred.  # noqa: E501

        :return: The preferred_during_scheduling_ignored_during_execution of this V1NodeAffinity.  # noqa: E501
        :rtype: list[V1PreferredSchedulingTerm]
        """
        return self._preferred_during_scheduling_ignored_during_execution

    @preferred_during_scheduling_ignored_during_execution.setter
    def preferred_during_scheduling_ignored_during_execution(self, preferred_during_scheduling_ignored_during_execution):
        """Sets the preferred_during_scheduling_ignored_during_execution of this V1NodeAffinity.

        The scheduler will prefer to schedule pods to nodes that satisfy the affinity expressions specified by this field, but it may choose a node that violates one or more of the expressions. The node that is most preferred is the one with the greatest sum of weights, i.e. for each node that meets all of the scheduling requirements (resource request, requiredDuringScheduling affinity expressions, etc.), compute a sum by iterating through the elements of this field and adding \"weight\" to the sum if the node matches the corresponding matchExpressions; the node(s) with the highest sum are the most preferred.  # noqa: E501

        :param preferred_during_scheduling_ignored_during_execution: The preferred_during_scheduling_ignored_during_execution of this V1NodeAffinity.  # noqa: E501
        :type preferred_during_scheduling_ignored_during_execution: list[V1PreferredSchedulingTerm]
        """

        self._preferred_during_scheduling_ignored_during_execution = preferred_during_scheduling_ignored_during_execution

    @property
    def required_during_scheduling_ignored_during_execution(self):
        """Gets the required_during_scheduling_ignored_during_execution of this V1NodeAffinity.  # noqa: E501


        :return: The required_during_scheduling_ignored_during_execution of this V1NodeAffinity.  # noqa: E501
        :rtype: V1NodeSelector
        """
        return self._required_during_scheduling_ignored_during_execution

    @required_during_scheduling_ignored_during_execution.setter
    def required_during_scheduling_ignored_during_execution(self, required_during_scheduling_ignored_during_execution):
        """Sets the required_during_scheduling_ignored_during_execution of this V1NodeAffinity.


        :param required_during_scheduling_ignored_during_execution: The required_during_scheduling_ignored_during_execution of this V1NodeAffinity.  # noqa: E501
        :type required_during_scheduling_ignored_during_execution: V1NodeSelector
        """

        self._required_during_scheduling_ignored_during_execution = required_during_scheduling_ignored_during_execution

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1NodeAffinity):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1NodeAffinity):
            return True

        return self.to_dict() != other.to_dict()
